#include "Scaling.inc"

// GS2x shader - by guest(r) (FX port of the 2xGLSL shader)
// guest.r@gmail.com
// license: GNU-GPL

// PS: Manny thanks to Ryan A. Nunn :)


string name : NAME = "GS2x";

string combineTechique : COMBINETECHNIQUE =  "GS2x_SHADER";

// vertex shader linkin' structure

struct VERTEX_STUFF
{
  float4  coord : POSITION;
  float2  CT    : TEXCOORD0;
  float2  UL    : TEXCOORD1;
  float2  UR    : TEXCOORD2;
  float2  DL    : TEXCOORD3;
  float2  DR    : TEXCOORD4;
};


// **VS**

VERTEX_STUFF GS_VERTEX (float3 Position : POSITION, float2 TexCoord : TEXCOORD0)
{
  VERTEX_STUFF OUT = (VERTEX_STUFF)0;
  
  float dx = TexelSize.x*0.5;
  float dy = TexelSize.y*0.5;

  OUT.coord = mul(float4(Position, 1), WorldViewProjection);
  OUT.CT = TexCoord;
  OUT.UL = TexCoord + float2(-dx,-dy);
  OUT.UR = TexCoord + float2( dx,-dy);
  OUT.DL = TexCoord + float2(-dx, dy);
  OUT.DR = TexCoord + float2( dx, dy);
  return OUT;
}


// **PS**

float4 GS_FRAGMENT ( in VERTEX_STUFF VAR ) : COLOR

{
  half3 c11 = tex2D(SourceSampler, VAR.CT).xyz;
  half3 c00 = tex2D(SourceSampler, VAR.UL).xyz;
  half3 c20 = tex2D(SourceSampler, VAR.UR).xyz;
  half3 c02 = tex2D(SourceSampler, VAR.DL).xyz;
  half3 c22 = tex2D(SourceSampler, VAR.DR).xyz;
	
  float3 dt = float3(1,1,1);
  
  float k00=dot(abs(c00-c11),dt);
  float k20=dot(abs(c20-c11),dt);
  float k02=dot(abs(c02-c11),dt);
  float k22=dot(abs(c22-c11),dt);

  float md1 = abs(k00-k22);
  float md2 = abs(k02-k20);

  float w1 = md2*k22;
  float w2 = md1*k02;
  float w3 = md2*k00;
  float w4 = md1*k20;

  float t1 = w1+w3;
  float t2 = w2+w4;

  float ww = max(t1,t2)+0.001;

  return float4((w1*c00+w2*c20+w3*c22+w4*c02+ww*c11)/(t1+t2+ww),0);
}



technique GS2x_SHADER
{
    pass P0
    {
		VertexShader = compile vs_2_0 GS_VERTEX();
		PixelShader  = compile ps_2_0 GS_FRAGMENT();
		AlphaBlendEnable = FALSE;
		ColorWriteEnable = RED|GREEN|BLUE|ALPHA;
		SRGBWRITEENABLE = FALSE;
    }  
}


